import java.util.*;
public class DijkstraPath extends Dijkstra {
    int[] via;
    public DijkstraPath(double[][] map) { super(map); } // コンストラクタ
    public void solve(int src,int dst) {
	this.src = src;		// 始点を記憶しておく
	boolean[] fixed = new boolean[nTown]; // 最短距離が確定したかどうか
	distance = new double[nTown]; // 各都市までの最短距離
	via = new int[nTown];	// その都市へ最短経路で到達する直前の都市
	for (int i=0; i<nTown; i++) { // 初期化する
	    distance[i] = Double.MAX_VALUE; // 最短距離の初期値は無限遠
	    fixed[i] = false;	// 最短距離はまだ確定していない
	    via[i] = -1;	// 直前の都市の初期値は-1
	}
	distance[src] = 0;	// 出発地点までの距離を0とする
	while (true) {
	    // 未確定の中で最も近い都市を求める
	    int marked = minIndex(distance,fixed);
	    if (marked < 0) return; // 全都市が確定した場合
	    if (distance[marked]==Double.MAX_VALUE) return; // 非連結グラフ
	    fixed[marked] = true; // その都市までの最短距離は確定となる
	    if (marked == dst) return; // 目的地までの距離が確定したので終了
	    for (int j=0; j<nTown; j++) { // 隣の都市(i)について
		if (map[marked][j]>0 && !fixed[j]) { // 未確定ならば
		    // markedが表す都市を経由した距離を求める
		    double newDistance = distance[marked]+map[marked][j];
		    // marked経由の距離が今までよりも小さければ更新する
		    if (newDistance < distance[j]) {
			distance[j] = newDistance;
			via[j] = marked; // 直前の都市も記憶しておく
		    }
		}
	    }
	}
    }
    public int[] getPath(int dst) { // srcからdstまでの経路を返す
	ArrayList<Integer> v = new ArrayList<Integer>();
	for (int i=dst; i >= 0; i=via[i]) {
	    v.add(new Integer(i));
	}
	int[] route = new int[v.size()]; // 経路用の配列を用意して
	for (int i=0; i<route.length; i++) // ベクタを整数配列に直す
	    route[i] = v.get(v.size()-1-i).intValue(); // vは逆順なので
	return route;
    }
}
